﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;

using System.Collections;
using System.Collections.Generic;
using Marlboro.Web.Common;

namespace Marlboro.Web.Extension
{
    public class BuildJson<TEntity, IdType>
    {
        /// <summary>
        /// MVC 无分页 Json数据构造
        /// </summary>
        /// <param name="entities">对象实体集合</param>
        /// <param name="idFunc">泛型委托，传入匿名函数lambda 传入TEntity 返回 IdType</param>
        /// <param name="propertyFuncs">泛型委托集合 用法同上</param>
        public BuildJson(
            List<TEntity> entities,
            Func<TEntity, IdType> idFunc,
            params Func<TEntity, string>[] propertyFuncs
            )
        {
            //无分页，实际分页为1
            this.ActualPageIndex = 1;
            //总页数为1
            this.TotalPageCount = 1;
            //总行数则为对象实体集合的实体个数
            this.TotalRowCount = entities.Count;
            //声明一个List来存入构造出的对象(每一行代表一个实体)
            this.Rows = new List<object>();

            //循环传入的实体集合
            foreach (TEntity entity in entities)
            {
                //声明一个obj的list来存放泛型委托集合中每个泛型委托返回的值
                List<string> obj = new List<string>();
                //循环泛型委托集合
                foreach (Func<TEntity, string> propertyFunc in propertyFuncs)
                {
                    //将每一个泛型委托 的返回值传入obj
                    obj.Add(propertyFunc(entity));
                }
                //将每个实体的ID 以及其他属性存入一个row当中
                this.Rows.Add(new
                {
                    id = idFunc(entity),
                    cell = obj.ToArray()
                });
            }

        }

        //实际页数
        public int ActualPageIndex { get; set; }
        //总页数
        public int TotalPageCount { get; set; }
        //总行数
        public int TotalRowCount { get; set; }
        //接收每个实体的集合
        public List<object> Rows { get; private set; }


        //数据组装
        public object Build()
        {
            return new
            {
                total = this.TotalPageCount,
                page = this.ActualPageIndex,
                records = this.TotalRowCount,
                rows = this.Rows
            };
        }


        /// <summary>
        /// MVC 分页 Json数据构造
        /// </summary>
        /// <param name="pagerResponse"></param>
        /// <param name="idFunc"></param>
        /// <param name="propertyFuncs"></param>
        public BuildJson(
            PagerResponse<TEntity> pagerResponse,
            Func<TEntity, IdType> idFunc,
            params Func<TEntity, string>[] propertyFuncs)
        {
            this.TotalRowCount = pagerResponse.TotalRowCount;
            this.ActualPageIndex = pagerResponse.ActualPageIndex;
            this.TotalPageCount = pagerResponse.PageCount;

            this.Rows = new List<object>();

            foreach (TEntity entity in pagerResponse.Entities)
            {
                List<string> obj = new List<string>();
                foreach (Func<TEntity, string> propertyFunc in propertyFuncs)
                {
                    obj.Add(propertyFunc(entity));
                }

                this.Rows.Add(
                    new
                    {
                        id = idFunc(entity),
                        cell = obj.ToArray()
                    });
            }
        }
    }
}
